@@ -3,8 +3,10 @@ import traceback
 import logging
 from pathlib import Path
 from time import time
+from urllib.parse import urlparse, ParseResult
 
-from flask import g, redirect, render_template, request, url_for
+from flask import g, redirect, render_template, request, url_for, request
+from flask.wrappers import Response, Request
 from flask_babel import lazy_gettext as _
 from flask_wtf.csrf import CSRFError
 from werkzeug.exceptions import MethodNotAllowed, NotFound
@@ -88,7 +90,7 @@ def server_specter_error(se):
         return {"error": str(se)}
     flash(str(se), "error")
     try:
-        app.specter.wallet_manager.update()
+        app.specter.wallet_manager.update(comment="via server_specter_error")
     except SpecterError as se:
         flash(str(se), "error")
     if request.method == "POST":
@@ -106,7 +108,10 @@ def server_notFound_error(e):
     app.logger.error(error_msg)
     if request.headers.get("Accept") == "application/json":
         return {"error": error_msg}
-    return render_template("500.jinja", error=e), 404
+    return (
+        render_template("500.jinja", error=e, version=app.specter.version.current),
+        404,
+    )
 
 
 @app.errorhandler(Exception)
@@ -118,7 +123,12 @@ def server_error(e):
     app.logger.error(trace)
     if request.headers.get("Accept") == "application/json":
         return {"error": error_msg}
-    return render_template("500.jinja", error=e, traceback=trace), 500
+    return (
+        render_template(
+            "500.jinja", error=e, traceback=trace, version=app.specter.version.current
+        ),
+        500,
+    )
 
 
 @app.errorhandler(BrokenCoreConnectionException)
@@ -181,7 +191,12 @@ def server_error_405(e):
     app.logger.error(trace)
     if request.headers.get("Accept") == "application/json":
         return {"error": error_msg}
-    flash(_("Session expired. Please refresh and try again."), "error")
+    flash(
+        _(
+            "Method not allowed. Probably the session expired. Please refresh and try again."
+        ),
+        "error",
+    )
     return redirect(request.url)
 
 
@@ -207,9 +222,26 @@ def execute_service_manager_hook():
 
 
 @app.after_request
-def slow_request_detection_stop(response):
+def slow_request_detection_stop(response: Response):
     try:
         diff = time() - g.start
+        if (
+            not request.url.__contains__("static")
+            and app.config["ENABLE_OWN_REQUEST_LOGGING"]
+        ):
+            # This is supposed to replace the annoying ENABLE_WERZEUG_REQUEST_LOGGING
+            # but only for non static Requests which are indeed interesting
+            # Original looks like this:
+            # [2023-02-10 14:15:49,716] INFO in _internal: 127.0.0.1 - - [10/Feb/2023 14:15:49] "GET /ext/devhelp/static/devhelp/img/orange-wrench.png HTTP/1.1" 304 -
+            url: ParseResult = urlparse(request.url)
+            logger.info(  # commas for being able to split up more easily afterwards
+                "{}, {: <40}, {}, took, {: >4}, ms".format(
+                    request.method,
+                    url.path + url.params + url.query + url.fragment,
+                    response.status_code,
+                    int(diff * 1000),
+                )
+            )
     except Exception as e:
         app.logger.exception(e)
         return response
