@@ -15,13 +15,14 @@ from sys import exit
 from urllib.parse import urlparse
 
 import requests
-from cryptoadvance.specter.devices.device_types import DeviceTypes
+from cryptoadvance.specter.devices.bitcoin_core import BitcoinCore, BitcoinCoreWatchOnly
 from cryptoadvance.specter.services.service_encrypted_storage import (
     ServiceEncryptedStorageManager,
     ServiceUnencryptedStorageManager,
 )
 from requests.exceptions import ConnectionError
 from stem import SocketError
+from stem.connection import AuthenticationFailure
 from stem.control import Controller
 from urllib3.exceptions import NewConnectionError
 
@@ -29,16 +30,23 @@ from .helpers import clean_psbt, deep_update, get_asset_label, is_liquid
 from .internal_node import InternalNode
 from .liquid.rpc import LiquidRPC
 from .managers.config_manager import ConfigManager
+from .managers.device_manager import DeviceManager
 from .managers.node_manager import NodeManager
 from .managers.otp_manager import OtpManager
-from .managers.service_manager import ServiceManager
+from .managers.service_manager import ExtensionManager
 from .managers.user_manager import UserManager
 from .managers.wallet_manager import WalletManager
 from .node import Node
 from .persistence import read_json_file, write_json_file, write_node
 from .process_controller.bitcoind_controller import BitcoindPlainController
-from .rpc import BitcoinRPC, RpcError, get_default_datadir
+from .rpc import (
+    BitcoinRPC,
+    RpcError,
+    get_default_datadir,
+)
+from .managers.service_manager import ExtensionManager
 from .services.service import devstatus_alpha, devstatus_beta, devstatus_prod
+from .services import callbacks
 from .specter_error import ExtProcTimeoutException, SpecterError
 from .tor_daemon import TorDaemonController
 from .user import User
@@ -78,6 +86,7 @@ class Specter:
         if not os.path.isdir(data_folder):
             os.makedirs(data_folder)
 
+        self.call_functions_at_cleanup_on_exit = []
         self.data_folder = data_folder
         self._config = config
         self._internal_bitcoind_version = internal_bitcoind_version
@@ -126,10 +135,8 @@ class Specter:
             )
         except SpecterError as e:
             if str(e).endswith("does not exist!"):
-                logger.warning(
-                    f"Current Node doesn't exist. Switching over to node {self.node_manager.DEFAULT_ALIAS}."
-                )
-                self.update_active_node(self.node_manager.DEFAULT_ALIAS)
+                if len(self.node_manager.nodes) > 0:
+                    self.update_active_node(next(iter(self.nodes.values())).alias)
             else:
                 raise e
 
@@ -156,13 +163,14 @@ class Specter:
         # also loads and checks wallets for all users
         try:
             self.check(check_all=True)
-
-            if os.path.isfile(self.torbrowser_path):
-                self.tor_daemon.start_tor_daemon()
         except Exception as e:
             logger.exception(e)
 
-        self.update_tor_controller()
+        if self.tor_type == "builtin" and os.path.isfile(self.torbrowser_path):
+            self.tor_daemon.start_tor_daemon()
+        if self.tor_type != "disabled":
+            self.update_tor_controller()
+
         self.checker = Checker(lambda: self.check(check_all=True), desc="health")
         if self._checker_threads:
             self.checker.start()
@@ -188,6 +196,7 @@ class Specter:
             signal.signal(signal.SIGINT, self.cleanup_on_exit)
             # This is for kill $pid --> SIGTERM
             signal.signal(signal.SIGTERM, self.cleanup_on_exit)
+        # a list of functions that are called at cleanup_on_exit taking in each signum, frame
 
     def cleanup_on_exit(self, signum=0, frame=0):
         if self._tor_daemon:
@@ -198,6 +207,9 @@ class Specter:
             if not node.external_node:
                 node.stop()
 
+        for f in self.call_functions_at_cleanup_on_exit:
+            f(signum, frame)
+
         logger.info("Closing Specter after cleanup")
         # For some reason we need to explicitely exit here. Otherwise it will hang
         exit(0)
@@ -224,24 +236,22 @@ class Specter:
             period = 600
         if hasattr(self, "checker") and self.checker.period != period:
             self.checker.period = period
+            logger.info(
+                f"Changing checker frequency: Every {period} seconds due to (rpc is None: {self.rpc is None} or initialblockdownload: { self.node.info.get('initialblockdownload', True) } ) "
+            )
 
         if not check_all:
             # find proper user
             user = self.user_manager.get_user(user)
             user.check()
         else:
+            u: User
             for u in self.user_manager.users:
                 u.check()
 
     @property
     def node(self) -> Node:
-        try:
-            return self.node_manager.active_node
-        except SpecterError as e:
-            logger.error("SpecterError while accessing active_node")
-            logger.exception(e)
-            self.update_active_node(list(self.node_manager.nodes.values())[0].alias)
-            return self.node_manager.active_node
+        return self.node_manager.active_node
 
     @property
     def default_node(self):
@@ -342,9 +352,7 @@ class Specter:
         self.setup_status[software_name]["stage_progress"] = -1
 
     def get_setup_status(self, software_name):
-        if software_name == "bitcoind":
-            installed = os.path.isfile(self.bitcoind_path)
-        elif software_name == "torbrowser":
+        if software_name == "torbrowser":
             installed = os.path.isfile(self.torbrowser_path)
         else:
             installed = False
@@ -394,13 +402,13 @@ class Specter:
     # mark
     def update_tor_control_port(self, tor_control_port, user):
         """set the control port of the tor daemon"""
-        if self.config_manager.update_tor_control_port:
-            self.update_tor_controller()
+        self.config_manager.update_tor_control_port(tor_control_port, user)
 
     # mark
     def generate_torrc_password(self, overwrite=False):
         self.config_manager.generate_torrc_password(overwrite)
 
+    # This is only used for the custom Tor setup, the built-in setup uses TorDaemonController
     def update_tor_controller(self):
         if "torrc_password" not in self.config:
             # Will be missing if the user did not go through the built-in Tor setup
@@ -419,6 +427,9 @@ class Specter:
         except SocketError as e:
             logger.warning(f"Failed to connect to Tor control port. Error: {e}")
             self._tor_controller = None
+        except AuthenticationFailure as e:
+            logger.warning(f"Failed to authenticate for Tor. Error: {e}")
+            self._tor_controller = None
 
     @property
     def tor_daemon(self):
@@ -440,14 +451,9 @@ class Specter:
 
     @property
     def tor_controller(self):
-        if self._tor_controller:
+        if hasattr(self, "_tor_controller"):
             return self._tor_controller
-        self.update_tor_controller()
-        if self._tor_controller:
-            return self._tor_controller
-        raise SpecterError(
-            "Failed to connect to the Tor daemon. Make sure ControlPort is properly configured."
-        )
+        return None
 
     # mark
     def update_hwi_bridge_url(self, url, user):
@@ -563,7 +569,7 @@ class Specter:
 
     @property
     def active_node_alias(self):
-        return self.user_config.get("active_node_alias", "default")
+        return self.user_config.get("active_node_alias", None)
 
     @property
     def explorer(self):
@@ -608,7 +614,7 @@ class Specter:
 
     @property
     def tor_type(self):
-        return self.user_config.get("tor_type", "builtin")
+        return self.user_config.get("tor_type", "disabled")
 
     @property
     def proxy_url(self):
@@ -651,31 +657,31 @@ class Specter:
         return self.user_config.get("alt_symbol", "BTC")
 
     @property
-    def admin(self):
+    def admin(self) -> User:
         for u in self.user_manager.users:
             if u.is_admin:
                 return u
 
     @property
-    def user(self):
+    def user(self) -> User:
         return self.user_manager.user
 
     @property
-    def config_manager(self):
+    def config_manager(self) -> ConfigManager:
         if not hasattr(self, "_config_manager"):
             self._config_manager = ConfigManager(self.data_folder)
         return self._config_manager
 
     @property
-    def device_manager(self):
+    def device_manager(self) -> DeviceManager:
         return self.user.device_manager
 
     @property
-    def wallet_manager(self):
+    def wallet_manager(self) -> WalletManager:
         return self.user.wallet_manager
 
     @property
-    def otp_manager(self):
+    def otp_manager(self) -> OtpManager:
         if not hasattr(self, "_otp_manager"):
             self._otp_manager = OtpManager(self.data_folder)
         return self._otp_manager
@@ -727,8 +733,8 @@ class Specter:
                     data.compress_type = zipfile.ZIP_DEFLATED
                     device = device.json
                     # Exporting the bitcoincore hot wallet as watchonly
-                    if device["type"] == DeviceTypes.BITCOINCORE:
-                        device["type"] = DeviceTypes.BITCOINCORE_WATCHONLY
+                    if device["type"] == BitcoinCore.device_type:
+                        device["type"] = BitcoinCoreWatchOnly.device_type
                     zf.writestr(
                         "devices/{}.json".format(device["alias"]), json.dumps(device)
                     )
@@ -743,7 +749,7 @@ class Specter:
         old_internal_rpc = self.config.get("internal_node", None)
         if old_internal_rpc and os.path.isfile(self.bitcoind_path):
             internal_node = InternalNode(
-                "Specter Bitcoin",
+                "Specter Bitcoin internal",
                 "specter_bitcoin",
                 old_internal_rpc.get("autodetect", False),
                 old_internal_rpc.get("datadir", get_default_datadir()),
@@ -753,7 +759,7 @@ class Specter:
                 old_internal_rpc.get("host", "localhost"),
                 old_internal_rpc.get("protocol", "http"),
                 os.path.join(
-                    os.path.join(self.data_folder, "nodes"), "specter_bitcoin.json"
+                    os.path.join(self.data_folder, "nodes"), "bitcoin_core.json"
                 ),
                 self,
                 self.bitcoind_path,
@@ -774,7 +780,7 @@ class Specter:
         if old_rpc:
             node = Node(
                 "Bitcoin Core",
-                "default",
+                "bitcoin_core",
                 old_rpc.get("autodetect", True),
                 old_rpc.get("datadir", get_default_datadir()),
                 old_rpc.get("user", ""),
@@ -782,16 +788,21 @@ class Specter:
                 old_rpc.get("port", None),
                 old_rpc.get("host", "localhost"),
                 old_rpc.get("protocol", "http"),
-                os.path.join(os.path.join(self.data_folder, "nodes"), "default.json"),
+                os.path.join(
+                    os.path.join(self.data_folder, "nodes"), "bitcoin_core.json"
+                ),
                 "BTC",
                 self,
             )
             logger.info(f"persisting {node} in migrate_old_node_format")
             write_node(
                 node,
-                os.path.join(os.path.join(self.data_folder, "nodes"), "default.json"),
+                os.path.join(
+                    os.path.join(self.data_folder, "nodes"), "bitcoin_core.json"
+                ),
             )
             del self.config["rpc"]
+            self.config_manager.update_active_node("bitcoin_core")
         self._save()
 
 
